@@ -37,6 +37,10 @@ from setuptools.command import sdist
 log.set_verbosity(log.INFO)
 TRUE_VALUES = ('true', '1', 'yes')
 REQUIREMENTS_FILES = ('requirements.txt', 'tools/pip-requires')
+TEST_REQUIREMENTS_FILES = ('test-requirements.txt', 'tools/test-requires')
+# part of the standard library starting with 2.7
+# adding it to the requirements list screws distro installs
+BROKEN_ON_27 = ('argparse', 'importlib')
 
 
 def append_text_list(config, key, text_list):
@@ -112,16 +116,26 @@ def canonicalize_emails(changelog, mapping):
     return changelog
 
 
+def _any_existing(file_list):
+    return [f for f in file_list if os.path.exists(f)]
+
+
 # Get requirements from the first file that exists
 def get_reqs_from_files(requirements_files):
-    for requirements_file in requirements_files:
-        if os.path.exists(requirements_file):
-            with open(requirements_file, 'r') as fil:
-                return fil.read().split('\n')
+    for requirements_file in _any_existing(requirements_files):
+        with open(requirements_file, 'r') as fil:
+            return fil.read().split('\n')
     return []
 
 
-def parse_requirements(requirements_files=REQUIREMENTS_FILES):
+def parse_requirements(requirements_files=None):
+
+    if requirements_files is None:
+        files = os.environ.get("PBR_REQUIREMENTS_FILES")
+        if files:
+            requirements_files = tuple(f.strip() for f in files.split(','))
+        else:
+            requirements_files = REQUIREMENTS_FILES
 
     def egg_fragment(match):
         # take a versioned egg fragment and return a
@@ -133,6 +147,10 @@ def parse_requirements(requirements_files=REQUIREMENTS_FILES):
 
     requirements = []
     for line in get_reqs_from_files(requirements_files):
+        # Ignore comments
+        if (not line.strip()) or line.startswith('#'):
+            continue
+
         # For the requirements list, we need to inject only the portion
         # after egg= so that distutils knows the package it's looking for
         # such as:
@@ -152,9 +170,7 @@ def parse_requirements(requirements_files=REQUIREMENTS_FILES):
         # -f lines are for index locations, and don't get used here
         elif re.match(r'\s*-f\s+', line):
             pass
-        # argparse is part of the standard library starting with 2.7
-        # adding it to the requirements list screws distro installs
-        elif line == 'argparse' and sys.version_info >= (2, 7):
+        elif line in BROKEN_ON_27 and sys.version_info >= (2, 7):
             pass
         else:
             requirements.append(line)
@@ -220,8 +236,12 @@ def write_git_changelog(git_dir=None, dest_dir=os.path.curdir,
     should_skip = get_boolean_option(option_dict, 'skip_changelog',
                                      'SKIP_WRITE_GIT_CHANGELOG')
     if not should_skip:
-        log.info('[pbr] Writing ChangeLog')
         new_changelog = os.path.join(dest_dir, 'ChangeLog')
+        # If there's already a ChangeLog and it's not writable, just use it
+        if (os.path.exists(new_changelog)
+                and not os.access(new_changelog, os.W_OK)):
+            return
+        log.info('[pbr] Writing ChangeLog')
         if git_dir is None:
             git_dir = _get_git_directory()
         if git_dir:
@@ -238,10 +258,14 @@ def generate_authors(git_dir=None, dest_dir='.', option_dict=dict()):
     should_skip = get_boolean_option(option_dict, 'skip_authors',
                                      'SKIP_GENERATE_AUTHORS')
     if not should_skip:
-        log.info('[pbr] Generating AUTHORS')
-        jenkins_email = 'jenkins@review'
         old_authors = os.path.join(dest_dir, 'AUTHORS.in')
         new_authors = os.path.join(dest_dir, 'AUTHORS')
+        # If there's already a ChangeLog and it's not writable, just use it
+        if (os.path.exists(new_authors)
+                and not os.access(new_authors, os.W_OK)):
+            return
+        log.info('[pbr] Generating AUTHORS')
+        jenkins_email = 'jenkins@review'
         if git_dir is None:
             git_dir = _get_git_directory()
         if git_dir:
@@ -312,6 +336,90 @@ class LocalInstall(install.install):
         return du_install.install.run(self)
 
 
+def _newer_requires_files(egg_info_dir):
+    """Check to see if any of the requires files are newer than egg-info."""
+    for target, sources in (('requires.txt', REQUIREMENTS_FILES),
+                            ('test-requires.txt', TEST_REQUIREMENTS_FILES)):
+        target_path = os.path.join(egg_info_dir, target)
+        for src in _any_existing(sources):
+            if (not os.path.exists(target_path) or
+                    os.path.getmtime(target_path)
+                    < os.path.getmtime(os.path.join(egg_info_dir, src))):
+                return True
+    return False
+
+
+def _copy_test_requires_to(egg_info_dir):
+    """Copy the requirements file to egg-info/test-requires.txt."""
+    with open(os.path.join(egg_info_dir, 'test-requires.txt'), 'w') as dest:
+        for source in _any_existing(TEST_REQUIREMENTS_FILES):
+            dest.write(open(source, 'r').read().rstrip('\n') + '\n')
+
+
+class _PipInstallTestRequires(object):
+    """Mixin class to install test-requirements.txt before running tests."""
+
+    def install_test_requirements(self):
+
+        links = _make_links_args(_any_existing(TEST_REQUIREMENTS_FILES))
+        if self.distribution.tests_require:
+            _pip_install(links, self.distribution.tests_requires)
+
+    def pre_run(self):
+        self.egg_name = pkg_resources.safe_name(self.distribution.get_name())
+        self.egg_info = "%s.egg-info" % self.egg_name
+        if (not os.path.exists(self.egg_info) or
+                _newer_requires_files(self.egg_info)):
+            ei_cmd = self.get_finalized_command('egg_info')
+            ei_cmd.run()
+            self.install_test_requirements()
+            _copy_test_requires_to(self.egg_info)
+
+try:
+    from pbr import testr_command
+
+    class TestrTest(testr_command.Testr, _PipInstallTestRequires):
+        """Make setup.py test do the right thing."""
+
+        command_name = 'test'
+
+        def run(self):
+            self.pre_run()
+            # Can't use super - base class old-style class
+            testr_command.Testr.run(self)
+
+    _have_testr = True
+
+except ImportError:
+    _have_testr = False
+
+
+def have_testr():
+    return _have_testr
+
+try:
+    from nose import commands
+
+    class NoseTest(commands.nosetests, _PipInstallTestRequires):
+        """Fallback test runner if testr is a no-go."""
+
+        command_name = 'test'
+
+        def run(self):
+            self.pre_run()
+            # Can't use super - base class old-style class
+            commands.nosetests.run(self)
+
+    _have_nose = True
+
+except ImportError:
+    _have_nose = False
+
+
+def have_nose():
+    return _have_nose
+
+
 class LocalSDist(sdist.sdist):
     """Builds the ChangeLog and Authors files from VC first."""
 
